@@ -24,7 +24,7 @@
 #include <string.h>
 
 #include <glib.h>
-#include "message.h"
+#include "diacontext.h"
 #include "boundingbox.h"
 
 #ifdef G_OS_WIN32
@@ -81,6 +81,7 @@ calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
   real add_len;
   real angle;
   Point tmp;
+  real dist;
   ArrowType arrow_type = arrow->type;
   /* Otherwise line is drawn through arrow
    * head for some hollow arrow heads
@@ -88,6 +89,7 @@ calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
   if (linewidth == 0.0)
     linewidth = 0.0001;
 
+  dist = distance_point_point (from, to);
   /** Since some of the calculations are sensitive to small values,
    * ignore small arrowheads.  They won't be visible anyway.
    */
@@ -96,6 +98,9 @@ calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
     arrow_type = ARROW_NONE;
   }
 
+  /* default to non-moving arrow */
+  move_arrow->x = 0.0;
+  move_arrow->y = 0.0;
   /* First, we move the arrow head backwards.
    * This in most cases just accounts for the linewidth of the arrow.
    * In pointy arrows, this means we must look at the angle of the
@@ -108,18 +113,23 @@ calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
   case ARROW_FILLED_CONCAVE:
   case ARROW_BLANKED_CONCAVE:
   case ARROW_DOUBLE_HOLLOW_TRIANGLE:
-    if (arrow->width < 0.0000001) return;
-    angle = atan(arrow->length/(arrow->width/2));
+    if (arrow->width < 0.0000001)
+      angle = 75*2*G_PI/360.0; /* -> add_len=0 */
+    else
+      angle = atan(arrow->length/(arrow->width/2));
     if (angle < 75*2*G_PI/360.0) {
       add_len = .5*linewidth/cos(angle);
     } else {
       add_len = 0;
     }
 
-    *move_arrow = *to;
-    point_sub(move_arrow, from);
-    point_normalize(move_arrow);    
-    point_scale(move_arrow, add_len);
+    /* don't move arrow if it would change direction */
+    if (fabs(add_len) < dist) {
+      *move_arrow = *to;
+      point_sub(move_arrow, from);
+      point_normalize(move_arrow);    
+      point_scale(move_arrow, add_len);
+    }
     break;
   case ARROW_HALF_HEAD:
     if (arrow->width < 0.0000001) return;
@@ -130,10 +140,13 @@ calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
       add_len = 0;
     }
 
-    *move_arrow = *to;
-    point_sub(move_arrow, from);
-    point_normalize(move_arrow);    
-    point_scale(move_arrow, add_len);
+    /* don't move arrow if it would change direction */
+    if (fabs(add_len) < dist) {
+      *move_arrow = *to;
+      point_sub(move_arrow, from);
+      point_normalize(move_arrow);    
+      point_scale(move_arrow, add_len);
+    }
     break;
   case ARROW_FILLED_TRIANGLE:
   case ARROW_HOLLOW_ELLIPSE:
@@ -143,10 +156,13 @@ calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
   case ARROW_BLANKED_BOX:
     add_len = .5*linewidth;
 
-    *move_arrow = *to;
-    point_sub(move_arrow, from);
-    point_normalize(move_arrow);
-    point_scale(move_arrow, add_len);
+    /* don't move arrow if it would change direction */
+    if (fabs(add_len) < dist) {
+      *move_arrow = *to;
+      point_sub(move_arrow, from);
+      point_normalize(move_arrow);
+      point_scale(move_arrow, add_len);
+    }
     break;
   case ARROW_ONE_EXACTLY:
   case ARROW_ONE_OR_NONE:
@@ -254,7 +270,6 @@ calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
   case ARROW_INTEGRAL_SYMBOL:
     *move_line = *to;
     point_sub(move_line, from);
-    add_len = point_len(move_line);
     point_normalize(move_line);
     point_scale(move_line, arrow->length / 2);
     return;
@@ -387,7 +402,7 @@ draw_none_or_many(DiaRenderer *renderer, Point *to, Point *from,
   /* use the middle of the arrow */
 
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
  
@@ -464,7 +479,7 @@ draw_one_or_many(DiaRenderer *renderer, Point *to, Point *from,
   calculate_arrow(poly, to, from, length, width);
  
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
  
@@ -533,7 +548,7 @@ draw_crow_foot(DiaRenderer *renderer, Point *to, Point *from,
   calculate_crow(poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
 
   DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer,&poly[0],&poly[1],fg_color);
@@ -559,7 +574,7 @@ draw_lines(DiaRenderer *renderer, Point *to, Point *from,
   calculate_arrow(poly, to, from, length, width);
     
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
     
@@ -594,7 +609,7 @@ draw_fill_ellipse(DiaRenderer *renderer, Point *to, Point *from,
   Point vl,vt;
 
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
@@ -635,10 +650,9 @@ draw_fill_ellipse(DiaRenderer *renderer, Point *to, Point *from,
   point_copy_add_scaled(&bp[2].p1,&bp[1].p3,&vl,length / 4.0);
   point_copy_add_scaled(&bp[3].p2,&bp[3].p3,&vl,length / 4.0);
   if (bg_color) {
-    DIA_RENDERER_GET_CLASS(renderer)->fill_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),bg_color);
-    DIA_RENDERER_GET_CLASS(renderer)->draw_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),fg_color);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_beziergon(renderer,bp,sizeof(bp)/sizeof(bp[0]),bg_color,fg_color);
   } else {
-    DIA_RENDERER_GET_CLASS(renderer)->fill_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),fg_color);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_beziergon(renderer,bp,sizeof(bp)/sizeof(bp[0]),fg_color,NULL);
   }
 }
 
@@ -661,7 +675,7 @@ draw_empty_ellipse(DiaRenderer *renderer, Point *to, Point *from,
   Point disp;
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
@@ -760,7 +774,7 @@ draw_fill_box(DiaRenderer *renderer, Point *to, Point *from,
   real lw_factor,clength,cwidth;
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
@@ -777,10 +791,9 @@ draw_fill_box(DiaRenderer *renderer, Point *to, Point *from,
   calculate_box (poly, to, from, clength, cwidth);
 
   if (fg_color == bg_color) {
-    DIA_RENDERER_GET_CLASS(renderer)->fill_polygon(renderer, poly, 4, fg_color);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, fg_color, NULL);
   } else {
-    DIA_RENDERER_GET_CLASS(renderer)->fill_polygon(renderer, poly, 4, bg_color);
-    DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, fg_color);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, bg_color, fg_color);
   }
   DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer,&poly[4],&poly[5],fg_color);
 }
@@ -812,7 +825,7 @@ draw_fill_dot(DiaRenderer *renderer, Point *to, Point *from,
   real lw_factor,clength,cwidth;
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
@@ -869,7 +882,7 @@ draw_fill_dot(DiaRenderer *renderer, Point *to, Point *from,
     
     DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer,&dos,&doe,fg_color);
   } else {
-    DIA_RENDERER_GET_CLASS(renderer)->fill_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),bg_color);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_beziergon(renderer,bp,sizeof(bp)/sizeof(bp[0]),bg_color,NULL);
   }
   if (fg_color != bg_color) {
     DIA_RENDERER_GET_CLASS(renderer)->draw_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),fg_color);
@@ -898,7 +911,7 @@ draw_integral(DiaRenderer *renderer, Point *to, Point *from,
   Point vl,vt;
   Point bs,be, bs2,be2;
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
@@ -988,7 +1001,7 @@ draw_slashed(DiaRenderer *renderer, Point *to, Point *from,
   calculate_slashed (poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
@@ -1013,13 +1026,6 @@ calculate_halfhead(Point *poly, const Point *to, const Point *from,
   Point delta;
   Point orth_delta;
   real len;
-  real angle;
-
-  if (width > 0.0000001) {
-    angle = atan(length/(width/2));
-  } else {
-    angle = 0;
-  }
 
   delta = *to;
   point_sub(&delta, from);
@@ -1069,7 +1075,7 @@ draw_halfhead(DiaRenderer *renderer, Point *to, Point *from,
   calculate_halfhead(poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
     
@@ -1088,41 +1094,17 @@ draw_halfhead(DiaRenderer *renderer, Point *to, Point *from,
 static void
 draw_triangle(DiaRenderer *renderer, Point *to, Point *from,
 	      real length, real width, real linewidth,
-	      Color *fg_color, Color *bg_color)
+	      Color *bg_color, Color *fg_color)
 {
   Point poly[3];
 
   calculate_arrow(poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
 
-  DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 3, fg_color);
-}
-
-/** Draw a simple triangular arrow, with filled head.
- * @param renderer A renderer instance to draw into
- * @param to The point that the arrow points to.
- * @param from Where the arrow points from (e.g. end of stem)
- * @param length The length of the arrow
- * @param width The width of the arrow
- * @param linewidth The thickness of the lines used to draw the arrow.
- * @param color The color used for drawing the arrowhead.
- */
-static void
-fill_triangle(DiaRenderer *renderer, Point *to, Point *from,
-	      real length, real width, real linewidth,
-	      Color *fg_color, Color *bg_color)
-{
-  Point poly[3];
-
-  calculate_arrow(poly, to, from, length, width);
-  
-  DIA_RENDERER_GET_CLASS(renderer)->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
-
-  DIA_RENDERER_GET_CLASS(renderer)->fill_polygon(renderer, poly, 3, bg_color);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 3, bg_color, fg_color);
 }
 
 /** Calculate the points needed to draw a diamon arrowhead.
@@ -1187,18 +1169,18 @@ calculate_diamond(Point *poly, const Point *to, const Point *from,
 static void
 draw_diamond(DiaRenderer *renderer, Point *to, Point *from,
 	     real length, real width, real linewidth,
-	     Color *color)
+	     Color *fill, Color *stroke)
 {
   Point poly[4];
 
   calculate_diamond(poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
-  DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, color);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, fill, stroke);
 }
 
 /** Draw a right-hand part of a diamond arrowhead.
@@ -1220,37 +1202,13 @@ draw_half_diamond(DiaRenderer *renderer, Point *to, Point *from,
   calculate_diamond(poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
   DIA_RENDERER_GET_CLASS(renderer)->draw_polyline(renderer, poly+1, 3, fg_color);
 }
 
-/** Draw a filled diamond arrow head.
- * @param renderer A renderer instance to draw into
- * @param to The point that the arrow points to.
- * @param from Where the arrow points from (e.g. end of stem)
- * @param length The length of the arrow
- * @param width The width of the arrow
- * @param color The color used for drawing the arrowhead.
- */
-static void
-fill_diamond(DiaRenderer *renderer, Point *to, Point *from,
-	     real length, real width,
-	     Color *color)
-{
-  Point poly[4];
-
-  calculate_diamond(poly, to, from, length, width);
-  
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
-
-  DIA_RENDERER_GET_CLASS(renderer)->fill_polygon(renderer, poly, 4, color);
-}
-
 /** Calculate the points needed to draw a slashed-cross arrowhead.
  * @param poly A 6-element array to hold the points calculated:
  * @param to Where the arrow points to.
@@ -1320,7 +1278,7 @@ draw_slashed_cross(DiaRenderer *renderer, Point *to, Point *from,
   calculate_slashed_cross(poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
   
@@ -1384,7 +1342,7 @@ draw_backslash(DiaRenderer *renderer, Point *to, Point *from,
   calculate_backslash (poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
@@ -1410,7 +1368,7 @@ draw_cross(DiaRenderer *renderer, Point *to, Point *from,
   calculate_arrow(poly, to, from, length, width);
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
   
@@ -1467,34 +1425,13 @@ calculate_double_arrow(Point *second_to, Point *second_from,
 static void 
 draw_double_triangle(DiaRenderer *renderer, Point *to, Point *from,
 		     real length, real width, real linewidth, 
-		     Color *fg_color, Color *bg_color)
+		     Color *bg_color, Color *fg_color)
 {
   Point second_from, second_to;
   
-  draw_triangle(renderer, to, from, length, width, linewidth, fg_color, bg_color);
+  draw_triangle(renderer, to, from, length, width, linewidth, bg_color, fg_color);
   calculate_double_arrow(&second_to, &second_from, to, from, length+linewidth);
-  draw_triangle(renderer, &second_to, &second_from, length, width, linewidth, fg_color, bg_color);
-}
-
-/** Draw a filled double-triangle arrowhead.
- * @param renderer A renderer instance to draw into
- * @param to The point that the arrow points to.
- * @param from Where the arrow points from (e.g. end of stem)
- * @param length The length of the arrow
- * @param width The width of the arrow
- * @param linewidth The thickness of the lines used to draw the arrow.
- * @param color The color used for drawing the arrowhead.
- */
-static void 
-fill_double_triangle(DiaRenderer *renderer, Point *to, Point *from,
-		     real length, real width, real linewidth, 
-		     Color *fg_color, Color *bg_color)
-{
-  Point second_from, second_to;
-  
-  fill_triangle(renderer, to, from, length, width, linewidth, NULL, bg_color);
-  calculate_double_arrow(&second_to, &second_from, to, from, length);
-  fill_triangle(renderer, &second_to, &second_from, length, width, linewidth, NULL, bg_color);
+  draw_triangle(renderer, &second_to, &second_from, length, width, linewidth, bg_color, fg_color);
 }
 
 /** Calculate the points needed to draw a concave arrowhead.
@@ -1573,13 +1510,14 @@ draw_concave_triangle(DiaRenderer *renderer, Point *to, Point *from,
 
   calculate_concave(poly, to, from, length, width);
   
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
   if (fg_color == bg_color)
-    DIA_RENDERER_GET_CLASS(renderer)->fill_polygon(renderer, poly, 4, bg_color);
-  DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, fg_color);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, bg_color, bg_color);
+  else
+    DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 4, NULL, fg_color);
 }
 
 /** Draw a rounded (half-circle) arrowhead.
@@ -1602,31 +1540,32 @@ draw_rounded(DiaRenderer *renderer, Point *to, Point *from,
   real len, rayon;
   real rapport;
   real angle_start;
-  
+
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
-  
+
   delta = *from;
-  
+
   point_sub(&delta, to);	
-  
+
   len = sqrt(point_dot(&delta, &delta)); /* line length */
   rayon = (length / 2.0);
   if (len > 0.0) {
-    /* no length, no direction - but invalid coords */
+    /* otherwise no length, no direction - but invalid coords */
     rapport = rayon / len;
-  
+
     p.x += delta.x * rapport;
     p.y += delta.y * rapport;
-  }  
-  angle_start = 90.0 - dia_asin((p.y - to->y) / rayon) * (180.0 / 3.14);
-  if (p.x - to->x < 0) { angle_start = 360.0 - angle_start;  }
+  }
+  angle_start = 90.0 - dia_asin((p.y - to->y) / rayon) * (180.0 / G_PI);
+  if (p.x - to->x < 0) { angle_start = 360.0 - angle_start; }
   
   DIA_RENDERER_GET_CLASS(renderer)->draw_arc(renderer, &p, width, length, angle_start, angle_start - 180.0, fg_color);
   
   if (len > 0.0) {
+    /* scan-build complains about may be used uninitialized, but nothing is changing len since init */
     p.x += delta.x * rapport;
     p.y += delta.y * rapport;
   }
@@ -1652,11 +1591,9 @@ draw_open_rounded(DiaRenderer *renderer, Point *to, Point *from,
   Point p = *to;
   Point delta;
   real len, rayon;
-  real rapport;
   real angle_start;
-  Point p_line;
 
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
   
@@ -1668,25 +1605,16 @@ draw_open_rounded(DiaRenderer *renderer, Point *to, Point *from,
   rayon = (length / 2.0);
   if (len > 0.0) {
     /* no length, no direction - but invalid coords */
-    rapport = rayon / len;
-  
+    real rapport = rayon / len;
     p.x += delta.x * rapport;
     p.y += delta.y * rapport;
   }  
   angle_start = 90.0 - dia_asin((p.y - to->y) / rayon) * (180.0 / 3.14);
   if (p.x - to->x < 0) { angle_start = 360.0 - angle_start;  }
 
-  p_line = p;
-  if (len > 0.0) {
-    p_line.x += delta.x * rapport;
-    p_line.y += delta.y * rapport;
-  }
-  /*
-  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth * 2.0);
-  DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &p_line, to, bg_color);
-  */
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->draw_arc(renderer, &p, width, length, angle_start - 180.0, angle_start, fg_color);  
+  DIA_RENDERER_GET_CLASS(renderer)->draw_arc(renderer, &p, width, length,
+					     angle_start - 180.0, angle_start, fg_color);
 }
 
 /** Draw an arrowhead with a circle in front of a triangle, filled.
@@ -1711,12 +1639,12 @@ draw_filled_dot_n_triangle(DiaRenderer *renderer, Point *to, Point *from,
 
   DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
   DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
   
   delta = *from;
   
-  point_sub(&delta, to);	
+  point_sub(&delta, to);
   
   len = sqrt(point_dot(&delta, &delta)); /* line length */
  
@@ -1729,8 +1657,8 @@ draw_filled_dot_n_triangle(DiaRenderer *renderer, Point *to, Point *from,
     p_dot.x += delta.x * rapport;
     p_dot.y += delta.y * rapport;
   }
-  DIA_RENDERER_GET_CLASS(renderer)->fill_ellipse(renderer, &p_dot,
-						 width, width, fg_color);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_ellipse(renderer, &p_dot,
+						 width, width, fg_color, NULL);
   /* triangle */
   if (len > 0.0) {
     rapport = width / len;
@@ -1738,7 +1666,7 @@ draw_filled_dot_n_triangle(DiaRenderer *renderer, Point *to, Point *from,
     p_tri.y += delta.y * rapport;
   }
   calculate_arrow(poly, &p_tri, from, length, width);
-  DIA_RENDERER_GET_CLASS(renderer)->fill_polygon(renderer, poly, 3, fg_color);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_polygon(renderer, poly, 3, fg_color, NULL);
 }
 
 /** Draw an arrowhead that is simply three dots (ellipsis)
@@ -1772,7 +1700,7 @@ draw_three_dots(DiaRenderer *renderer, Point *to, Point *from,
   hole_width = width / 3 - dot_width;
   
   DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, linewidth);
-  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
 
   for (i = 0; i < 3; i++) {
     dot_from.x = to->x - i  * (dot_width + hole_width) * delta.x;
@@ -1782,45 +1710,43 @@ draw_three_dots(DiaRenderer *renderer, Point *to, Point *from,
     DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &dot_from, &dot_to, fg_color);
   }
 }
-
+/* hollow is still filed, but with background color */
 static void
 draw_hollow_triangle (DiaRenderer *renderer, Point *to, Point *from,
 		      real length, real width, real linewidth,
 		      Color *fg_color, Color *bg_color)
 {
-  fill_triangle(renderer, to, from, length, width, linewidth, NULL, bg_color);
-  draw_triangle(renderer, to, from, length, width, linewidth, fg_color, NULL);
+  draw_triangle(renderer, to, from, length, width, linewidth, bg_color, fg_color);
 }
+/* unfilled is no backround drawn */
 static void
 draw_filled_triangle (DiaRenderer *renderer, Point *to, Point *from,
 		      real length, real width, real linewidth,
 		      Color *fg_color, Color *bg_color)
 {
-  fill_triangle(renderer, to, from, length, width, linewidth, NULL, fg_color);
-  draw_triangle(renderer, to, from, length, width, linewidth, fg_color, NULL);
+  draw_triangle(renderer, to, from, length, width, linewidth, fg_color, fg_color);
 }
 static void
 draw_unfilled_triangle (DiaRenderer *renderer, Point *to, Point *from,
 		        real length, real width, real linewidth,
 		        Color *fg_color, Color *bg_color)
 {
-  draw_triangle(renderer, to, from, length, width, linewidth, fg_color, bg_color);
+  draw_triangle(renderer, to, from, length, width, linewidth, NULL, fg_color);
 }
+/* hollow is still filed, but with background color */
 static void
 draw_hollow_diamond (DiaRenderer *renderer, Point *to, Point *from,
 		     real length, real width, real linewidth,
 		     Color *fg_color, Color *bg_color)
 {
-  fill_diamond(renderer, to, from, length, width, bg_color);
-  draw_diamond(renderer, to, from, length, width, linewidth, fg_color);
+  draw_diamond(renderer, to, from, length, width, linewidth, bg_color, fg_color);
 }
 static void
 draw_filled_diamond (DiaRenderer *renderer, Point *to, Point *from,
 		     real length, real width, real linewidth,
 		     Color *fg_color, Color *bg_color)
 {
-  fill_diamond(renderer, to, from, length, width, fg_color);
-  draw_diamond(renderer, to, from, length, width, linewidth, fg_color);
+  draw_diamond(renderer, to, from, length, width, linewidth, fg_color, fg_color);
 }
 static void
 draw_filled_ellipse (DiaRenderer *renderer, Point *to, Point *from,
@@ -1866,15 +1792,14 @@ draw_double_hollow_triangle (DiaRenderer *renderer, Point *to, Point *from,
 		             real length, real width, real linewidth,
 		             Color *fg_color, Color *bg_color)
 {
-  fill_double_triangle(renderer, to, from, length+(linewidth/2), width, linewidth, fg_color, bg_color);
-  draw_double_triangle(renderer, to, from, length, width, linewidth, fg_color, bg_color);  
+  draw_double_triangle(renderer, to, from, length, width, linewidth, bg_color, fg_color);  
 }
 static void
 draw_double_filled_triangle (DiaRenderer *renderer, Point *to, Point *from,
 		             real length, real width, real linewidth,
 		             Color *fg_color, Color *bg_color)
 {
-  fill_double_triangle(renderer, to, from, length, width, linewidth, fg_color, fg_color);
+  draw_double_triangle(renderer, to, from, length, width, linewidth, fg_color, fg_color);
 }
 struct ArrowDesc {
   const char *name;
@@ -1895,47 +1820,47 @@ struct ArrowDesc {
 	        Color *fg_color, /* the main drawin color */
 	        Color *bg_color); /* not always used */
 } arrow_types[] =
-  {{N_("None"),ARROW_NONE},
-   {N_("Lines"),ARROW_LINES, calculate_arrow, draw_lines}, 
-   {N_("Hollow Triangle"), ARROW_HOLLOW_TRIANGLE, calculate_arrow, draw_hollow_triangle},
-   {N_("Filled Triangle"), ARROW_FILLED_TRIANGLE, calculate_arrow, draw_filled_triangle},
-   {N_("Unfilled Triangle"), ARROW_UNFILLED_TRIANGLE, calculate_arrow, draw_unfilled_triangle},
-   {N_("Hollow Diamond"),ARROW_HOLLOW_DIAMOND, calculate_diamond, draw_hollow_diamond},
-   {N_("Filled Diamond"),ARROW_FILLED_DIAMOND, calculate_diamond, draw_filled_diamond},
-   {N_("Half Diamond"), ARROW_HALF_DIAMOND, calculate_diamond, draw_half_diamond},
-   {N_("Half Head"), ARROW_HALF_HEAD, calculate_halfhead, draw_halfhead},
-   {N_("Slashed Cross"), ARROW_SLASHED_CROSS, calculate_slashed_cross, draw_slashed_cross},
-   {N_("Filled Ellipse"), ARROW_FILLED_ELLIPSE, calculate_ellipse, draw_filled_ellipse},
-   {N_("Hollow Ellipse"), ARROW_HOLLOW_ELLIPSE, calculate_ellipse, draw_fill_ellipse},
-   {N_("Filled Dot"), ARROW_FILLED_DOT, calculate_dot, draw_filled_dot},
-   {N_("Dimension Origin"),ARROW_DIMENSION_ORIGIN},
-   {N_("Blanked Dot"),ARROW_BLANKED_DOT, calculate_dot, draw_fill_dot},
-   {N_("Double Hollow Triangle"),ARROW_DOUBLE_HOLLOW_TRIANGLE, calculate_double_triangle, draw_double_hollow_triangle},
-   {N_("Double Filled Triangle"),ARROW_DOUBLE_FILLED_TRIANGLE, calculate_double_triangle, draw_double_filled_triangle},
-   {N_("Filled Dot and Triangle"), ARROW_FILLED_DOT_N_TRIANGLE},
-   {N_("Filled Box"), ARROW_FILLED_BOX, calculate_box, draw_filled_box},
-   {N_("Blanked Box"),ARROW_BLANKED_BOX, calculate_box, draw_fill_box},
-   {N_("Slashed"), ARROW_SLASH_ARROW, calculate_slashed, draw_slashed},
-   {N_("Integral Symbol"),ARROW_INTEGRAL_SYMBOL},
-   {N_("Crow Foot"), ARROW_CROW_FOOT, calculate_crow, draw_crow_foot},
-   {N_("Cross"),ARROW_CROSS, calculate_arrow, draw_cross},
-   {N_("1-or-many"),ARROW_ONE_OR_MANY},
-   {N_("0-or-many"),ARROW_NONE_OR_MANY},
-   {N_("1-or-0"),ARROW_ONE_OR_NONE},
-   {N_("1 exactly"),ARROW_ONE_EXACTLY},
-   {N_("Filled Concave"),ARROW_FILLED_CONCAVE, calculate_concave, draw_filled_concave},
-   {N_("Blanked Concave"),ARROW_BLANKED_CONCAVE, calculate_concave, draw_concave_triangle},
-   {N_("Round"), ARROW_ROUNDED},
-   {N_("Open Round"), ARROW_OPEN_ROUNDED},
-   {N_("Backslash"), ARROW_BACKSLASH, calculate_backslash, draw_backslash},
-   {N_("Infinite Line"),ARROW_THREE_DOTS},
+  {{NC_("Arrow", "None"),ARROW_NONE},
+   {NC_("Arrow", "Lines"),ARROW_LINES, calculate_arrow, draw_lines}, 
+   {NC_("Arrow", "Hollow Triangle"), ARROW_HOLLOW_TRIANGLE, calculate_arrow, draw_hollow_triangle},
+   {NC_("Arrow", "Filled Triangle"), ARROW_FILLED_TRIANGLE, calculate_arrow, draw_filled_triangle},
+   {NC_("Arrow", "Unfilled Triangle"), ARROW_UNFILLED_TRIANGLE, calculate_arrow, draw_unfilled_triangle},
+   {NC_("Arrow", "Hollow Diamond"),ARROW_HOLLOW_DIAMOND, calculate_diamond, draw_hollow_diamond},
+   {NC_("Arrow", "Filled Diamond"),ARROW_FILLED_DIAMOND, calculate_diamond, draw_filled_diamond},
+   {NC_("Arrow", "Half Diamond"), ARROW_HALF_DIAMOND, calculate_diamond, draw_half_diamond},
+   {NC_("Arrow", "Half Head"), ARROW_HALF_HEAD, calculate_halfhead, draw_halfhead},
+   {NC_("Arrow", "Slashed Cross"), ARROW_SLASHED_CROSS, calculate_slashed_cross, draw_slashed_cross},
+   {NC_("Arrow", "Filled Ellipse"), ARROW_FILLED_ELLIPSE, calculate_ellipse, draw_filled_ellipse},
+   {NC_("Arrow", "Hollow Ellipse"), ARROW_HOLLOW_ELLIPSE, calculate_ellipse, draw_fill_ellipse},
+   {NC_("Arrow", "Filled Dot"), ARROW_FILLED_DOT, calculate_dot, draw_filled_dot},
+   {NC_("Arrow", "Dimension Origin"),ARROW_DIMENSION_ORIGIN},
+   {NC_("Arrow", "Blanked Dot"),ARROW_BLANKED_DOT, calculate_dot, draw_fill_dot},
+   {NC_("Arrow", "Double Hollow Triangle"),ARROW_DOUBLE_HOLLOW_TRIANGLE, calculate_double_triangle, draw_double_hollow_triangle},
+   {NC_("Arrow", "Double Filled Triangle"),ARROW_DOUBLE_FILLED_TRIANGLE, calculate_double_triangle, draw_double_filled_triangle},
+   {NC_("Arrow", "Filled Dot and Triangle"), ARROW_FILLED_DOT_N_TRIANGLE},
+   {NC_("Arrow", "Filled Box"), ARROW_FILLED_BOX, calculate_box, draw_filled_box},
+   {NC_("Arrow", "Blanked Box"),ARROW_BLANKED_BOX, calculate_box, draw_fill_box},
+   {NC_("Arrow", "Slashed"), ARROW_SLASH_ARROW, calculate_slashed, draw_slashed},
+   {NC_("Arrow", "Integral Symbol"),ARROW_INTEGRAL_SYMBOL},
+   {NC_("Arrow", "Crow Foot"), ARROW_CROW_FOOT, calculate_crow, draw_crow_foot},
+   {NC_("Arrow", "Cross"),ARROW_CROSS, calculate_arrow, draw_cross},
+   {NC_("Arrow", "1-or-many"),ARROW_ONE_OR_MANY},
+   {NC_("Arrow", "0-or-many"),ARROW_NONE_OR_MANY},
+   {NC_("Arrow", "1-or-0"),ARROW_ONE_OR_NONE},
+   {NC_("Arrow", "1 exactly"),ARROW_ONE_EXACTLY},
+   {NC_("Arrow", "Filled Concave"),ARROW_FILLED_CONCAVE, calculate_concave, draw_filled_concave},
+   {NC_("Arrow", "Blanked Concave"),ARROW_BLANKED_CONCAVE, calculate_concave, draw_concave_triangle},
+   {NC_("Arrow", "Round"), ARROW_ROUNDED},
+   {NC_("Arrow", "Open Round"), ARROW_OPEN_ROUNDED},
+   {NC_("Arrow", "Backslash"), ARROW_BACKSLASH, calculate_backslash, draw_backslash},
+   {NC_("Arrow", "Infinite Line"),ARROW_THREE_DOTS},
    {NULL,0}
 };
 
 /** following the signature pattern of lib/boundingbox.h 
  * the arrow bounding box is added to the given rect 
  * @param arrow the arrow
- * @param linewidth arrows use the same line width
+ * @param line_width arrows use the same line width
  * @param to The point that the arrow points to.
  * @param from Where the arrow points from (e.g. end of stem)
  * @param rect the preintialized bounding box
@@ -2028,11 +1953,11 @@ arrow_draw(DiaRenderer *renderer, ArrowType type,
       break;
     }
   }
-  if ((type != ARROW_NONE) && (render_bounding_boxes) && (renderer->is_interactive)) {
+  if ((type != ARROW_NONE) && (render_bounding_boxes()) && (renderer->is_interactive)) {
     Arrow arrow = {type, length, width};
     Rectangle bbox = {0, };
     Point p1, p2;
-    Color col = {1.0, 0.0, 1.0};
+    Color col = { 1.0, 0.0, 1.0, 1.0 };
     
     arrow_bbox (&arrow, linewidth, to, from, &bbox);
 
@@ -2042,7 +1967,7 @@ arrow_draw(DiaRenderer *renderer, ArrowType type,
     p2.y = bbox.bottom;
 
     DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer,0.01);
-    DIA_RENDERER_GET_CLASS(renderer)->draw_rect(renderer, &p1, &p2, &col);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_rect(renderer, &p1, &p2, NULL, &col);
   }
 }
 
@@ -2053,10 +1978,10 @@ arrow_draw(DiaRenderer *renderer, ArrowType type,
  * legal head type.
  */
 static void
-sanitize_arrow(Arrow *arrow)
+sanitize_arrow(Arrow *arrow, DiaContext *ctx)
 {
-  if (arrow->type < 0 || arrow->type > MAX_ARROW_TYPE) {
-    message_warning(_("Arrow head of unknown type"));
+  if (arrow->type > MAX_ARROW_TYPE) {
+    dia_context_add_message(ctx, _("Arrow head of unknown type"));
     arrow->type = ARROW_NONE;
     arrow->width = DEFAULT_ARROW_WIDTH;
     arrow->length = DEFAULT_ARROW_LENGTH;
@@ -2064,8 +1989,8 @@ sanitize_arrow(Arrow *arrow)
 
   if (arrow->length < MIN_ARROW_DIMENSION ||
       arrow->width < MIN_ARROW_DIMENSION) {
-    message_warning(_("Arrow head of type %s has too small dimensions, removing.\n"),
-		    arrow_get_name_from_type(arrow->type));
+    dia_context_add_message(ctx, _("Arrow head of type %s has too small dimensions; removing.\n"),
+		            arrow_get_name_from_type(arrow->type));
     arrow->type = ARROW_NONE;
     arrow->width = DEFAULT_ARROW_WIDTH;
     arrow->length = DEFAULT_ARROW_LENGTH;
@@ -2081,14 +2006,14 @@ sanitize_arrow(Arrow *arrow)
  */
 void
 save_arrow(ObjectNode obj_node, Arrow *arrow, gchar *type_attribute,
-	   gchar *length_attribute, gchar *width_attribute)
+	   gchar *length_attribute, gchar *width_attribute, DiaContext *ctx)
 {
   data_add_enum(new_attribute(obj_node, type_attribute),
-		arrow->type);
+		arrow->type, ctx);
   data_add_real(new_attribute(obj_node, length_attribute),
-		arrow->length);
+		arrow->length, ctx);
   data_add_real(new_attribute(obj_node, width_attribute),
-		arrow->width);
+		arrow->width, ctx);
 }
 
 /** Load arrow information from three attributes.
@@ -2100,7 +2025,7 @@ save_arrow(ObjectNode obj_node, Arrow *arrow, gchar *type_attribute,
  */
 void
 load_arrow(ObjectNode obj_node, Arrow *arrow, gchar *type_attribute, 
-	   gchar *length_attribute, gchar *width_attribute)
+	   gchar *length_attribute, gchar *width_attribute, DiaContext *ctx)
 {
   AttributeNode attr;
 
@@ -2109,15 +2034,15 @@ load_arrow(ObjectNode obj_node, Arrow *arrow, gchar *type_attribute,
   arrow->width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, type_attribute);
   if (attr != NULL)
-    arrow->type = data_enum(attribute_first_data(attr));
+    arrow->type = data_enum(attribute_first_data(attr),ctx);
   attr = object_find_attribute(obj_node, length_attribute);
   if (attr != NULL)
-    arrow->length = data_real(attribute_first_data(attr));
+    arrow->length = data_real(attribute_first_data(attr),ctx);
   attr = object_find_attribute(obj_node, width_attribute);
   if (attr != NULL)
-    arrow->width = data_real(attribute_first_data(attr));
+    arrow->width = data_real(attribute_first_data(attr),ctx);
 
-  sanitize_arrow(arrow);
+  sanitize_arrow(arrow, ctx);
 }
 
 /** Returns the arrow type that corresponds to a given name.
@@ -2194,7 +2119,7 @@ get_arrow_names(void)
 const gchar *
 arrow_get_name_from_type(ArrowType type)
 {
-  if (type >= 0 && type < MAX_ARROW_TYPE) {
+  if (type < MAX_ARROW_TYPE) {
     return arrow_types[arrow_index_from_type(type)].name;
   }
   return _("unknown arrow");
