@@ -28,6 +28,7 @@
 #include <time.h>
 
 #include <pango/pango.h>
+#undef PANGO_DISABLE_DEPRECATED /* pango_ft_get_context */
 #ifdef HAVE_FREETYPE
 #include <pango/pangoft2.h>
 #endif
@@ -47,6 +48,10 @@
 
 static PangoContext* pango_context = NULL;
 
+/*!
+ * \brief Opaque type to represent a font
+ * \ingroup ObjectFonts
+ */
 struct _DiaFont 
 {
   GObject parent_instance;
@@ -144,6 +149,8 @@ dia_font_get_context()
     else {
 #  ifdef GDK_WINDOWING_WIN32
       dia_font_push_context(pango_win32_get_context ());
+#  elif defined(GDK_WINDOWING_QUARTZ) || defined(GDK_WINDOWING_X11)
+      dia_font_push_context (pango_font_map_create_context (pango_cairo_font_map_get_default ()));
 #  else
       g_warning ("dia_font_get_context() : not font context w/o display. Crashing soon.");
 #  endif
@@ -208,6 +215,7 @@ static void
 dia_pfd_set_height(PangoFontDescription* pfd, real height)
 { 
   /* ONLY place for the magic factor! */
+  g_return_if_fail (height > 0.0);
   pango_font_description_set_absolute_size(pfd, dcm_to_pdu(height) * 0.8);
 }
 
@@ -412,13 +420,13 @@ dia_font_get_style(const DiaFont* font)
   return style;
 }
 
-G_CONST_RETURN char*
+const char*
 dia_font_get_family(const DiaFont* font)
 {
   return pango_font_description_get_family(font->pfd);
 }
 
-G_CONST_RETURN PangoFontDescription *
+const PangoFontDescription *
 dia_font_get_description(const DiaFont* font)
 {
   return font->pfd;
@@ -445,7 +453,7 @@ dia_font_set_height(DiaFont* font, real height)
 }
 
 
-G_CONST_RETURN char*
+const char*
 dia_font_get_psfontname(const DiaFont *font)
 {
   /* This hack corrects a couple fonts that were misnamed in
@@ -535,7 +543,7 @@ static const WeightName weight_names[] = {
   {0,NULL}
 };
 
-G_CONST_RETURN char *
+const char *
 dia_font_get_weight_string(const DiaFont* font)
 {
     const WeightName* p;
@@ -575,7 +583,7 @@ static const SlantName slant_names[] = {
   { 0, NULL} 
 };
 
-G_CONST_RETURN char *
+const char *
 dia_font_get_slant_string(const DiaFont* font)
 {
   const SlantName* p;
@@ -594,11 +602,6 @@ dia_font_set_slant_from_string(DiaFont* font, const char* obli)
 {
   DiaFontSlant fo = DIA_FONT_NORMAL;
   const SlantName* p;
-
-  DiaFontStyle old_style;
-  DiaFontSlant old_fo;
-  old_style = dia_font_get_style(font);
-  old_fo = DIA_FONT_STYLE_GET_SLANT(old_style);
     
   for (p = slant_names; p->name != NULL; ++p) {
     if (0 == strncmp(obli,p->name,8)) {
@@ -829,7 +832,9 @@ dia_font_get_sizes(const char* string, DiaFont *font, real height,
     *width = 0.0;
   } else {
     /* take the bigger rectangle to avoid cutting of any part of the string */
-    *width = pdu_to_dcm(logical_rect.width > ink_rect.width ? logical_rect.width : ink_rect.width) / global_zoom_factor;
+    /* also respect that Dia is assuming to start from zero */
+    int full_width = ink_rect.width + ink_rect.x;
+    *width = pdu_to_dcm(logical_rect.width > full_width ? logical_rect.width : full_width) / global_zoom_factor;
   }
   return offsets;
 }
@@ -950,7 +955,7 @@ dia_font_new_from_legacy_name(const char* name)
   return retval;
 }
 
-G_CONST_RETURN char*
+const char*
 dia_font_get_legacy_name(const DiaFont *font)
 {
   const char* matched_name = NULL;
