@@ -60,7 +60,6 @@ struct _Orthflow {
   Handle text_handle;
 
   Text* text;
-  TextAttributes attrs;
   OrthflowType type;
   Point textpos; /* This is the master position, only overridden in load */
 };
@@ -78,15 +77,15 @@ struct _OrthflowChange {
   char*				text ;
 };
 
-Color orthflow_color_energy   = { 1.0f, 0.0f, 0.0f };
-Color orthflow_color_material = { 0.8f, 0.0f, 0.8f };
-Color orthflow_color_signal   = { 0.0f, 0.0f, 1.0f };
+Color orthflow_color_energy   = { 1.0f, 0.0f, 0.0f, 1.0f };
+Color orthflow_color_material = { 0.8f, 0.0f, 0.8f, 1.0f };
+Color orthflow_color_signal   = { 0.0f, 0.0f, 1.0f, 1.0f };
+
 
-  
 #define ORTHFLOW_WIDTH 0.1
 #define ORTHFLOW_MATERIAL_WIDTH 0.2
 #define ORTHFLOW_DASHLEN 0.4
-#define ORTHFLOW_FONTHEIGHT 0.6
+#define ORTHFLOW_FONTHEIGHT 0.8
 #define ORTHFLOW_ARROWLEN 0.8
 #define ORTHFLOW_ARROWWIDTH 0.5
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM2)
@@ -113,9 +112,8 @@ orthflow_get_props(Orthflow * orthflow, GPtrArray *props);
 static void
 orthflow_set_props(Orthflow * orthflow, GPtrArray *props);
 static void orthflow_save(Orthflow *orthflow, ObjectNode obj_node,
-			  const char *filename);
-static DiaObject *orthflow_load(ObjectNode obj_node, int version,
-			     const char *filename);
+			  DiaContext *ctx);
+static DiaObject *orthflow_load(ObjectNode obj_node, int version,DiaContext *ctx);
 static DiaMenu *orthflow_get_object_menu(Orthflow *orthflow, Point *clickedpoint) ;
 
 
@@ -131,11 +129,11 @@ static ObjectTypeOps orthflow_type_ops =
 
 DiaObjectType orthflow_type =
 {
-  "FS - Orthflow",		/* name */
+  "FS - Orthflow",  /* name */
   /* Version 0 had no autorouting and so shouldn't have it set by default. */
-  1,                      /* version */
-  (char **) orthflow_xpm,	/* pixmap */
-  &orthflow_type_ops		/* ops */
+  1,             /* version */
+  orthflow_xpm,	  /* pixmap */
+  &orthflow_type_ops /* ops */
 };
 
 static ObjectOps orthflow_ops = {
@@ -164,7 +162,7 @@ static PropEnumData prop_orthflow_type_data[] = {
 };
 
 static PropDescription orthflow_props[] = {
-  ELEMENT_COMMON_PROPERTIES,
+  ORTHCONN_COMMON_PROPERTIES,
   { "type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
     N_("Type:"), NULL, prop_orthflow_type_data },
   { "text", PROP_TYPE_TEXT, 0, NULL, NULL },
@@ -186,20 +184,19 @@ orthflow_describe_props(Orthflow *mes)
 }
 
 static PropOffset orthflow_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  ORTHCONN_COMMON_PROPERTIES_OFFSETS,
   { "type", PROP_TYPE_ENUM, offsetof(Orthflow, type) },
   { "text", PROP_TYPE_TEXT, offsetof (Orthflow, text) },
-  { "text_alignment", PROP_TYPE_ENUM, offsetof (Orthflow, attrs.alignment) },
-  { "text_font", PROP_TYPE_FONT, offsetof (Orthflow, attrs.font) },
-  { PROP_STDNAME_TEXT_HEIGHT, PROP_STDTYPE_TEXT_HEIGHT, offsetof (Orthflow, attrs.height) },
-  { "text_colour", PROP_TYPE_COLOUR, offsetof (Orthflow, attrs.color) },
+  { "text_alignment", PROP_TYPE_ENUM, offsetof(Orthflow,text),offsetof(Text,alignment) },
+  { "text_font", PROP_TYPE_FONT, offsetof(Orthflow,text),offsetof(Text,font) },
+  { PROP_STDNAME_TEXT_HEIGHT, PROP_STDTYPE_TEXT_HEIGHT, offsetof(Orthflow,text),offsetof(Text,height) },
+  { "text_colour", PROP_TYPE_COLOUR, offsetof(Orthflow,text),offsetof(Text,color) },
   { NULL, 0, 0 }
 };
 
 static void
 orthflow_get_props(Orthflow * orthflow, GPtrArray *props)
 {
-  text_get_attributes (orthflow->text, &orthflow->attrs);
   object_get_props_from_offsets(&orthflow->orth.object, 
                                 orthflow_offsets, props);
 }
@@ -209,7 +206,6 @@ orthflow_set_props(Orthflow *orthflow, GPtrArray *props)
 {
   object_set_props_from_offsets(&orthflow->orth.object, 
                                 orthflow_offsets, props);
-  apply_textattr_properties (props, orthflow->text, "text", &orthflow->attrs);
   orthflow_update_data(orthflow);
 }
 
@@ -364,18 +360,17 @@ orthflow_draw(Orthflow *orthflow, DiaRenderer *renderer)
   switch (orthflow->type) {
   case ORTHFLOW_SIGNAL:
     linewidth = ORTHFLOW_WIDTH;
-    renderer_ops->set_dashlength(renderer, ORTHFLOW_DASHLEN);
-    renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED, ORTHFLOW_DASHLEN);
     render_color = &orthflow_color_signal ;
     break ;
   case ORTHFLOW_MATERIAL:
     linewidth = ORTHFLOW_MATERIAL_WIDTH;
-    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
     render_color = &orthflow_color_material ;
     break ;
   case ORTHFLOW_ENERGY:
     linewidth = ORTHFLOW_WIDTH;
-    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
     render_color = &orthflow_color_energy ;
     break ;
   default:
@@ -419,11 +414,10 @@ orthflow_create(Point *startpoint,
   p = *startpoint ;
   p.y += 0.1 * ORTHFLOW_FONTHEIGHT ;
   orthflow->textpos = p;
-  font = dia_font_new_from_style(DIA_FONT_SANS, 0.8);
+  font = dia_font_new_from_style(DIA_FONT_SANS, ORTHFLOW_FONTHEIGHT);
 
-  orthflow->text = new_text("", font, 0.8, &p, &color_black, ALIGN_CENTER);
+  orthflow->text = new_text("", font, ORTHFLOW_FONTHEIGHT, &p, &color_black, ALIGN_CENTER);
   dia_font_unref(font);  
-  text_get_attributes(orthflow->text, &orthflow->attrs);
 
 #if 0
   if ( orthflow_default_label ) {
@@ -539,18 +533,18 @@ orthflow_update_data(Orthflow *orthflow)
 
 
 static void
-orthflow_save(Orthflow *orthflow, ObjectNode obj_node, const char *filename)
+orthflow_save(Orthflow *orthflow, ObjectNode obj_node, DiaContext *ctx)
 {
-  orthconn_save(&orthflow->orth, obj_node);
+  orthconn_save(&orthflow->orth, obj_node, ctx);
 
   data_add_text(new_attribute(obj_node, "text"),
-		orthflow->text) ;
+		orthflow->text, ctx) ;
   data_add_int(new_attribute(obj_node, "type"),
-	       orthflow->type);
+	       orthflow->type, ctx);
 }
 
 static DiaObject *
-orthflow_load(ObjectNode obj_node, int version, const char *filename)
+orthflow_load(ObjectNode obj_node, int version, DiaContext *ctx)
 {
   Orthflow *orthflow;
   AttributeNode attr;
@@ -567,16 +561,22 @@ orthflow_load(ObjectNode obj_node, int version, const char *filename)
   obj->type = &orthflow_type;
   obj->ops = &orthflow_ops;
 
-  orthconn_load(orth, obj_node);
+  orthconn_load(orth, obj_node, ctx);
 
   orthflow->text = NULL;
   attr = object_find_attribute(obj_node, "text");
   if (attr != NULL)
-    orthflow->text = data_text(attribute_first_data(attr));
+    orthflow->text = data_text(attribute_first_data(attr), ctx);
+  else { /* paranoid */
+    DiaFont *font = dia_font_new_from_style(DIA_FONT_SANS, ORTHFLOW_FONTHEIGHT);
+
+    orthflow->text = new_text("", font, ORTHFLOW_FONTHEIGHT, &obj->position, &color_black, ALIGN_CENTER);
+    dia_font_unref(font);
+  }
 
   attr = object_find_attribute(obj_node, "type");
   if (attr != NULL)
-    orthflow->type = (OrthflowType)data_int(attribute_first_data(attr));
+    orthflow->type = (OrthflowType)data_int(attribute_first_data(attr), ctx);
 
   orthflow->text_handle.id = HANDLE_MOVE_TEXT;
   orthflow->text_handle.type = HANDLE_MINOR_CONTROL;
