@@ -35,8 +35,6 @@
 #include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
-#include "widgets.h"
-#include "message.h"
 #include "properties.h"
 
 #include "pixmaps/ellipse.xpm"
@@ -68,8 +66,9 @@ struct _Ellipse {
   real dashlength;
 
   Text *text;
-  TextAttributes attrs;
   real padding;
+
+  TextFitting text_fitting;
 };
 
 typedef struct _EllipseProperties {
@@ -99,8 +98,8 @@ static PropDescription *ellipse_describe_props(Ellipse *ellipse);
 static void ellipse_get_props(Ellipse *ellipse, GPtrArray *props);
 static void ellipse_set_props(Ellipse *ellipse, GPtrArray *props);
 
-static void ellipse_save(Ellipse *ellipse, ObjectNode obj_node, const char *filename);
-static DiaObject *ellipse_load(ObjectNode obj_node, int version, const char *filename);
+static void ellipse_save(Ellipse *ellipse, ObjectNode obj_node, DiaContext *ctx);
+static DiaObject *ellipse_load(ObjectNode obj_node, int version,DiaContext *ctx);
 
 static ObjectTypeOps ellipse_type_ops =
 {
@@ -113,11 +112,10 @@ static ObjectTypeOps ellipse_type_ops =
 
 DiaObjectType fc_ellipse_type =
 {
-  "Flowchart - Ellipse",  /* name */
-  0,                 /* version */
-  (char **) ellipse_xpm, /* pixmap */
-
-  &ellipse_type_ops      /* ops */
+  "Flowchart - Ellipse", /* name */
+  0,                  /* version */
+  ellipse_xpm,         /* pixmap */
+  &ellipse_type_ops       /* ops */
 };
 
 static ObjectOps ellipse_ops = {
@@ -153,6 +151,7 @@ static PropDescription ellipse_props[] = {
   PROP_STD_TEXT_HEIGHT,
   PROP_STD_TEXT_COLOUR,
   PROP_STD_TEXT_ALIGNMENT,
+  PROP_STD_TEXT_FITTING,
   PROP_STD_SAVED_TEXT,
   
   { NULL, 0, 0, NULL, NULL, NULL, 0}
@@ -176,17 +175,17 @@ static PropOffset ellipse_offsets[] = {
     offsetof(Ellipse, line_style), offsetof(Ellipse, dashlength) },
   { "padding", PROP_TYPE_REAL, offsetof(Ellipse, padding) },
   {"text",PROP_TYPE_TEXT,offsetof(Ellipse,text)},
-  {"text_font",PROP_TYPE_FONT,offsetof(Ellipse,attrs.font)},
-  {PROP_STDNAME_TEXT_HEIGHT,PROP_STDTYPE_TEXT_HEIGHT,offsetof(Ellipse,attrs.height)},
-  {"text_colour",PROP_TYPE_COLOUR,offsetof(Ellipse,attrs.color)},
-  {"text_alignment",PROP_TYPE_ENUM,offsetof(Ellipse,attrs.alignment)},
+  {"text_font",PROP_TYPE_FONT,offsetof(Ellipse,text),offsetof(Text,font)},
+  {PROP_STDNAME_TEXT_HEIGHT,PROP_STDTYPE_TEXT_HEIGHT,offsetof(Ellipse,text),offsetof(Text,height)},
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Ellipse,text),offsetof(Text,color)},
+  {"text_alignment",PROP_TYPE_ENUM,offsetof(Ellipse,text),offsetof(Text,alignment)},
+  {PROP_STDNAME_TEXT_FITTING,PROP_TYPE_ENUM,offsetof(Ellipse,text_fitting)},
   { NULL, 0, 0 },
 };
 
 static void
 ellipse_get_props(Ellipse *ellipse, GPtrArray *props)
 {
-  text_get_attributes(ellipse->text,&ellipse->attrs);
   object_get_props_from_offsets(&ellipse->element.object,
                                 ellipse_offsets,props);
 }
@@ -196,7 +195,6 @@ ellipse_set_props(Ellipse *ellipse, GPtrArray *props)
 {
   object_set_props_from_offsets(&ellipse->element.object,
                                 ellipse_offsets,props);
-  apply_textattr_properties(props,ellipse->text,"text",&ellipse->attrs);
   ellipse_update_data(ellipse, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 }
 
@@ -232,6 +230,8 @@ ellipse_radius(Ellipse *ellipse, real px, real py)
   px *= px;
   py *= py;
 
+  if (px <= 0.0 && py <= 0.0)
+    return 0; /* avoid division by zero */
   scale = w2 * h2 / (4*h2*px + 4*w2*py);
   return sqrt((px + py)*scale);
 }
@@ -241,16 +241,12 @@ ellipse_distance_from(Ellipse *ellipse, Point *point)
 {
   Element *elem = &ellipse->element;
   Point c;
-  real dist, rad;
 
   c.x = elem->corner.x + elem->width / 2;
   c.y = elem->corner.y + elem->height/ 2;
-  dist = distance_point_point(point, &c);
-  rad = ellipse_radius(ellipse, point->x, point->y) + ellipse->border_width/2;
 
-  if (dist <= rad)
-    return 0;
-  return dist - rad;
+  return distance_ellipse_point (&c, elem->width, elem->height,
+				 ellipse->border_width, point);
 }
 
 static void
@@ -269,11 +265,18 @@ ellipse_move_handle(Ellipse *ellipse, Handle *handle,
 		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   AnchorShape horiz = ANCHOR_MIDDLE, vert = ANCHOR_MIDDLE;
+  Point corner;
+  real width, height;
 
   assert(ellipse!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
+  /* remember ... */
+  corner = ellipse->element.corner;
+  width = ellipse->element.width;
+  height = ellipse->element.height;
+
   element_move_handle(&ellipse->element, handle->id, to, cp, 
 		      reason, modifiers);
 
@@ -299,6 +302,9 @@ ellipse_move_handle(Ellipse *ellipse, Handle *handle,
   }
   ellipse_update_data(ellipse, horiz, vert);
 
+  if (width != ellipse->element.width || height != ellipse->element.height)
+    return element_change_new (&corner, width, height, &ellipse->element);
+
   return NULL;
 }
 
@@ -327,21 +333,15 @@ ellipse_draw(Ellipse *ellipse, DiaRenderer *renderer)
   center.x = elem->corner.x + elem->width/2;
   center.y = elem->corner.y + elem->height/2;
 
-  if (ellipse->show_background) {
-    renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  
-    renderer_ops->fill_ellipse(renderer, &center,
-				elem->width, elem->height,
-				&ellipse->inner_color);
-  }
-
+  if (ellipse->show_background)
+    renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);  
   renderer_ops->set_linewidth(renderer, ellipse->border_width);
-  renderer_ops->set_linestyle(renderer, ellipse->line_style);
-  renderer_ops->set_dashlength(renderer, ellipse->dashlength);
+  renderer_ops->set_linestyle(renderer, ellipse->line_style, ellipse->dashlength);
   renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
-  renderer_ops->draw_ellipse(renderer, &center,
+  renderer_ops->draw_ellipse (renderer, &center,
 			      elem->width, elem->height,
+			      (ellipse->show_background) ? &ellipse->inner_color : NULL,
 			      &ellipse->border_color);
 
   text_draw(ellipse->text, renderer);
@@ -386,7 +386,9 @@ ellipse_update_data(Ellipse *ellipse, AnchorShape horiz, AnchorShape vert)
   radius1 = ellipse_radius(ellipse, p.x, p.y) - ellipse->border_width/2;
   radius2 = distance_point_point(&c, &p);
   
-  if (radius1 < radius2) {
+  if (   (radius1 < radius2 && ellipse->text_fitting == TEXTFIT_WHEN_NEEDED)
+      /* stop infinite resizing with 5% tolerance obsvered with _test_movement */
+      || (fabs(1.0 - radius2 / radius1) > 0.05 && ellipse->text_fitting == TEXTFIT_ALWAYS)) {
     /* increase size of the ellipse while keeping its aspect ratio */
     elem->width  *= radius2 / radius1;
     elem->height *= radius2 / radius1;
@@ -494,9 +496,11 @@ ellipse_create(Point *startpoint,
   p.y += elem->height / 2.0 + font_height / 2;
   ellipse->text = new_text("", font, font_height, &p, &ellipse->border_color,
 			   ALIGN_CENTER);
-  text_get_attributes(ellipse->text,&ellipse->attrs);
   dia_font_unref(font);
   
+  /* new default: let the user decide the size */
+  ellipse->text_fitting = TEXTFIT_WHEN_NEEDED;
+
   element_init(elem, 8, NUM_CONNECTIONS);
 
   for (i=0;i<NUM_CONNECTIONS;i++) {
@@ -523,40 +527,45 @@ ellipse_destroy(Ellipse *ellipse)
 }
 
 static void
-ellipse_save(Ellipse *ellipse, ObjectNode obj_node, const char *filename)
+ellipse_save(Ellipse *ellipse, ObjectNode obj_node, DiaContext *ctx)
 {
-  element_save(&ellipse->element, obj_node);
+  element_save(&ellipse->element, obj_node, ctx);
 
   if (ellipse->border_width != 0.1)
     data_add_real(new_attribute(obj_node, "border_width"),
-		  ellipse->border_width);
+		  ellipse->border_width, ctx);
   
   if (!color_equals(&ellipse->border_color, &color_black))
     data_add_color(new_attribute(obj_node, "border_color"),
-		   &ellipse->border_color);
+		   &ellipse->border_color, ctx);
   
   if (!color_equals(&ellipse->inner_color, &color_white))
     data_add_color(new_attribute(obj_node, "inner_color"),
-		   &ellipse->inner_color);
+		   &ellipse->inner_color, ctx);
   
-  data_add_boolean(new_attribute(obj_node, "show_background"), ellipse->show_background);
+  data_add_boolean(new_attribute(obj_node, "show_background"),
+		   ellipse->show_background, ctx);
 
   if (ellipse->line_style != LINESTYLE_SOLID)
     data_add_enum(new_attribute(obj_node, "line_style"),
-		  ellipse->line_style);
+		  ellipse->line_style, ctx);
 
   if (ellipse->line_style != LINESTYLE_SOLID &&
       ellipse->dashlength != DEFAULT_LINESTYLE_DASHLEN)
     data_add_real(new_attribute(obj_node, "dashlength"),
-                  ellipse->dashlength);
+                  ellipse->dashlength, ctx);
 
-  data_add_real(new_attribute(obj_node, "padding"), ellipse->padding);
+  data_add_real(new_attribute(obj_node, "padding"), ellipse->padding, ctx);
 
-  data_add_text(new_attribute(obj_node, "text"), ellipse->text);
+  data_add_text(new_attribute(obj_node, "text"), ellipse->text, ctx);
+
+  if (ellipse->text_fitting != TEXTFIT_WHEN_NEEDED)
+    data_add_enum(new_attribute(obj_node, PROP_STDNAME_TEXT_FITTING),
+		  ellipse->text_fitting, ctx);
 }
 
 static DiaObject *
-ellipse_load(ObjectNode obj_node, int version, const char *filename)
+ellipse_load(ObjectNode obj_node, int version,DiaContext *ctx)
 {
   Ellipse *ellipse;
   Element *elem;
@@ -571,47 +580,55 @@ ellipse_load(ObjectNode obj_node, int version, const char *filename)
   obj->type = &fc_ellipse_type;
   obj->ops = &ellipse_ops;
 
-  element_load(elem, obj_node);
+  element_load(elem, obj_node, ctx);
   
   ellipse->border_width = 0.1;
   attr = object_find_attribute(obj_node, "border_width");
   if (attr != NULL)
-    ellipse->border_width =  data_real( attribute_first_data(attr) );
+    ellipse->border_width =  data_real(attribute_first_data(attr), ctx);
 
   ellipse->border_color = color_black;
   attr = object_find_attribute(obj_node, "border_color");
   if (attr != NULL)
-    data_color(attribute_first_data(attr), &ellipse->border_color);
+    data_color(attribute_first_data(attr), &ellipse->border_color, ctx);
   
   ellipse->inner_color = color_white;
   attr = object_find_attribute(obj_node, "inner_color");
   if (attr != NULL)
-    data_color(attribute_first_data(attr), &ellipse->inner_color);
+    data_color(attribute_first_data(attr), &ellipse->inner_color, ctx);
   
   ellipse->show_background = TRUE;
   attr = object_find_attribute(obj_node, "show_background");
   if (attr != NULL)
-    ellipse->show_background = data_boolean( attribute_first_data(attr) );
+    ellipse->show_background = data_boolean( attribute_first_data(attr), ctx);
 
   ellipse->line_style = LINESTYLE_SOLID;
   attr = object_find_attribute(obj_node, "line_style");
   if (attr != NULL)
-    ellipse->line_style =  data_enum( attribute_first_data(attr) );
+    ellipse->line_style =  data_enum(attribute_first_data(attr), ctx);
 
   ellipse->dashlength = DEFAULT_LINESTYLE_DASHLEN;
   attr = object_find_attribute(obj_node, "dashlength");
   if (attr != NULL)
-    ellipse->dashlength = data_real(attribute_first_data(attr));
+    ellipse->dashlength = data_real(attribute_first_data(attr), ctx);
 
   ellipse->padding = default_properties.padding;
   attr = object_find_attribute(obj_node, "padding");
   if (attr != NULL)
-    ellipse->padding =  data_real( attribute_first_data(attr) );
+    ellipse->padding =  data_real(attribute_first_data(attr), ctx);
   
   ellipse->text = NULL;
   attr = object_find_attribute(obj_node, "text");
   if (attr != NULL)
-    ellipse->text = data_text(attribute_first_data(attr));
+    ellipse->text = data_text(attribute_first_data(attr), ctx);
+  else
+    ellipse->text = new_text_default(&obj->position, &ellipse->border_color, ALIGN_CENTER);
+
+  /* old default: only growth, manual shrink */
+  ellipse->text_fitting = TEXTFIT_WHEN_NEEDED;
+  attr = object_find_attribute(obj_node, PROP_STDNAME_TEXT_FITTING);
+  if (attr != NULL)
+    ellipse->text_fitting = data_enum(attribute_first_data(attr), ctx);
 
   element_init(elem, 8, NUM_CONNECTIONS);
 
