@@ -30,6 +30,7 @@
 #include "diarenderer.h"
 #include "attributes.h"
 #include "diamenu.h"
+#include "properties.h"
 
 #include "pixmaps/bus.xpm"
 
@@ -85,9 +86,8 @@ static DiaObject *bus_copy(Bus *bus);
 static PropDescription *bus_describe_props(Bus *bus);
 static void bus_get_props(Bus *bus, GPtrArray *props);
 static void bus_set_props(Bus *bus, GPtrArray *props);
-static void bus_save(Bus *bus, ObjectNode obj_node, const char *filename);
-static DiaObject *bus_load(ObjectNode obj_node, int version,
-			const char *filename);
+static void bus_save(Bus *bus, ObjectNode obj_node, DiaContext *ctx);
+static DiaObject *bus_load(ObjectNode obj_node, int version, DiaContext *ctx);
 static DiaMenu *bus_get_object_menu(Bus *bus, Point *clickedpoint);
 
 static ObjectChange *
@@ -109,15 +109,7 @@ DiaObjectType bus_type =
 {
   "Network - Bus",   /* name */
   0,                  /* version */
-  (char **) bus_xpm,  /* pixmap */
-  &bus_type_ops       /* ops */
-};
-
-DiaObjectType bus_type_std = /* Old mistake, left for backwards compatibility */
-{
-  "Standard - Bus",   /* name */
-  0,                  /* version */
-  (char **) bus_xpm,  /* pixmap */
+  (const char **) bus_xpm,  /* pixmap */
   &bus_type_ops       /* ops */
 };
 
@@ -140,7 +132,7 @@ static ObjectOps bus_ops = {
 };
 
 static PropDescription bus_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  CONNECTION_COMMON_PROPERTIES,
   PROP_STD_LINE_COLOUR,
   PROP_DESC_END
 };
@@ -154,7 +146,7 @@ bus_describe_props(Bus *bus)
 }
 
 static PropOffset bus_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  CONNECTION_COMMON_PROPERTIES_OFFSETS,
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Bus, line_color) },
   { NULL, 0, 0 }
 };
@@ -207,24 +199,18 @@ bus_move_handle(Bus *bus, Handle *handle,
 {
   Connection *conn = &bus->connection;
   Point *endpoints;
-  static real *parallel=NULL;
-  static real *perp=NULL;
-  static int max_num=0;
+  real *parallel=NULL;
+  real *perp=NULL;
   Point vhat, vhatperp;
   Point u;
   real vlen, vlen2;
   real len_scale;
   int i;
+  const int num_handles = bus->num_handles; /* const to help scan-build */
 
-  if (bus->num_handles>max_num) {
-    if (parallel!=NULL) {
-      g_free(parallel);
-      g_free(perp);
-    }
-    parallel = g_malloc(sizeof(real)*bus->num_handles);
-    perp = g_malloc(sizeof(real)*bus->num_handles);
-    max_num = bus->num_handles;
-  }
+  /* code copied to Misc/tree.c */
+  parallel = (real *)g_alloca (num_handles * sizeof(real));
+  perp = (real *)g_alloca (num_handles * sizeof(real));
 
   if (handle->id == HANDLE_BUS) {
     handle->pos = *to;
@@ -240,7 +226,7 @@ bus_move_handle(Bus *bus, Handle *handle,
     
     vhatperp.x = -vhat.y;
     vhatperp.y =  vhat.x;
-    for (i=0;i<bus->num_handles;i++) {
+    for (i=0;i<num_handles;i++) {
       u = bus->handles[i]->pos;
       point_sub(&u, &endpoints[0]);
       parallel[i] = point_dot(&vhat, &u);
@@ -260,7 +246,7 @@ bus_move_handle(Bus *bus, Handle *handle,
     point_normalize(&vhat);
     vhatperp.x = -vhat.y;
     vhatperp.y =  vhat.x;
-    for (i=0;i<bus->num_handles;i++) {
+    for (i=0;i<num_handles;i++) {
       if (bus->handles[i]->connected_to == NULL) {
 	u = vhat;
 	point_scale(&u, parallel[i]*len_scale);
@@ -319,7 +305,7 @@ bus_draw(Bus *bus, DiaRenderer *renderer)
   endpoints = &bus->real_ends[0];
   
   renderer_ops->set_linewidth(renderer, LINE_WIDTH);
-  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID, 0.0);
   renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   renderer_ops->draw_line(renderer,
@@ -631,24 +617,24 @@ bus_get_object_menu(Bus *bus, Point *clickedpoint)
 }
 
 static void
-bus_save(Bus *bus, ObjectNode obj_node, const char *filename)
+bus_save(Bus *bus, ObjectNode obj_node, DiaContext *ctx)
 {
   int i;
   AttributeNode attr;
 
-  connection_save(&bus->connection, obj_node);
+  connection_save(&bus->connection, obj_node, ctx);
   
-  data_add_color( new_attribute(obj_node, "line_color"), &bus->line_color);
+  data_add_color( new_attribute(obj_node, "line_color"), &bus->line_color, ctx);
 
   attr = new_attribute(obj_node, "bus_handles");
   
   for (i=0;i<bus->num_handles;i++) {
-    data_add_point(attr, &bus->handles[i]->pos);
+    data_add_point(attr, &bus->handles[i]->pos, ctx);
   }
 }
 
 static DiaObject *
-bus_load(ObjectNode obj_node, int version, const char *filename)
+bus_load(ObjectNode obj_node, int version,DiaContext *ctx)
 {
   Bus *bus;
   Connection *conn;
@@ -667,7 +653,7 @@ bus_load(ObjectNode obj_node, int version, const char *filename)
   obj->type = &bus_type;
   obj->ops = &bus_ops;
 
-  connection_load(conn, obj_node);
+  connection_load(conn, obj_node, ctx);
 
   attr = object_find_attribute(obj_node, "bus_handles");
 
@@ -686,7 +672,7 @@ bus_load(ObjectNode obj_node, int version, const char *filename)
     bus->handles[i]->type = HANDLE_MINOR_CONTROL;
     bus->handles[i]->connect_type = HANDLE_CONNECTABLE_NOBREAK;
     bus->handles[i]->connected_to = NULL;
-    data_point(data, &bus->handles[i]->pos);
+    data_point(data, &bus->handles[i]->pos, ctx);
     obj->handles[2+i] = bus->handles[i];
 
     data = data_next(data);
@@ -695,7 +681,7 @@ bus_load(ObjectNode obj_node, int version, const char *filename)
   bus->line_color = color_black;
   attr = object_find_attribute(obj_node, "line_color");
   if (attr != NULL)
-    data_color(attribute_first_data(attr), &bus->line_color);
+    data_color(attribute_first_data(attr), &bus->line_color, ctx);
 
   extra->start_trans = 
     extra->end_trans = 
