@@ -33,7 +33,6 @@
 #include <glib/gstdio.h>
 
 #include "intl.h"
-#include "message.h"
 #include "geometry.h"
 #include "diarenderer.h"
 #include "filter.h"
@@ -56,7 +55,7 @@ Point extent_min, extent_max;
 Point limit_min, limit_max;
 
 /* maximum line length */
-#define DXF_LINE_LENGTH 256
+#define DXF_LINE_LENGTH 257
 
 typedef struct _DxfLayerData 
 {
@@ -71,32 +70,60 @@ typedef struct _DxfData
     char value[DXF_LINE_LENGTH];
 } DxfData;
 
-gboolean import_dxf(const gchar *filename, DiagramData *dia, void* user_data);
-gboolean read_dxf_codes(FILE *filedxf, DxfData *data);
-DiaObject *read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-DiaObject *read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-DiaObject *read_entity_ellipse_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-DiaObject *read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-DiaObject *read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-DiaObject *read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_entity_measurement_dxf(FILE *filedxf, DxfData *data,
-                                 DiagramData *dia);
-void read_entity_scale_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_entity_textsize_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_entity_mesurement_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_table_layer_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_section_header_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_section_classes_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_section_tables_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_section_entities_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-void read_section_blocks_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
-Layer *layer_find_by_name(char *layername, DiagramData *dia);
-LineStyle get_dia_linestyle_dxf(char *dxflinestyle);
+static gboolean import_dxf(const gchar *filename, DiagramData *dia, DiaContext *ctx, void* user_data);
+static gboolean read_dxf_codes(FILE *filedxf, DxfData *data);
+static DiaObject *read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static DiaObject *read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static DiaObject *read_entity_ellipse_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static DiaObject *read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static DiaObject *read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static DiaObject *read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_entity_measurement_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_entity_scale_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_entity_textsize_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_table_layer_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_section_header_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_section_classes_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_section_tables_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_section_entities_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static void read_section_blocks_dxf(FILE *filedxf, DxfData *data, DiagramData *dia);
+static Layer *layer_find_by_name(char *layername, DiagramData *dia);
+static LineStyle get_dia_linestyle_dxf(char *dxflinestyle);
+
+GHashTable *_color_by_layer_ht = NULL;
+
+static void
+_dxf_color_set_by_layer (const Layer *layer, int color_index)
+{
+  if (!_color_by_layer_ht) /* lazy creation */
+    _color_by_layer_ht = g_hash_table_new (g_direct_hash, g_direct_equal); 
+  g_hash_table_insert (_color_by_layer_ht, (void *)layer, GINT_TO_POINTER (color_index));
+} 
+static int
+_dxf_color_get_by_layer (const Layer *layer)
+{
+  int color_index;
+
+  if (!_color_by_layer_ht)
+    return 0;
+  color_index = GPOINTER_TO_INT (g_hash_table_lookup (_color_by_layer_ht, layer));
+  if (color_index > 0)
+    return color_index;
+  return 0;
+}
+static void
+_color_init_from_rgb (Color *color, RGB_t rgb)
+{
+  color->red   = rgb.r / 255.0;
+  color->green = rgb.g / 255.0;
+  color->blue  = rgb.b / 255.0;
+  color->alpha = 1.0;
+}
 
 /* returns the layer with the given name */
 /* TODO: merge this with other layer code? */
-Layer *
+static Layer *
 layer_find_by_name(char *layername, DiagramData *dia) 
 {
     Layer *matching_layer, *layer;
@@ -123,7 +150,7 @@ layer_find_by_name(char *layername, DiagramData *dia)
 
 /* returns the matching dia linestyle for a given dxf linestyle */
 /* if no matching style is found, LINESTYLE solid is returned as a default */
-LineStyle 
+static LineStyle
 get_dia_linestyle_dxf(char *dxflinestyle) 
 {
     if (strcmp(dxflinestyle, "DASHED") == 0)
@@ -138,17 +165,8 @@ get_dia_linestyle_dxf(char *dxflinestyle)
     return LINESTYLE_SOLID;
 }
 
-static PropDescription dxf_prop_descs[] = {
-    { "start_point", PROP_TYPE_POINT },
-    { "end_point", PROP_TYPE_POINT },
-    { "line_colour", PROP_TYPE_COLOUR },
-    { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH },
-    { "line_style", PROP_TYPE_LINESTYLE},
-    PROP_DESC_END};
-
-
 /* reads a line entity from the dxf file and creates a line object in dia*/
-DiaObject *
+static DiaObject *
 read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
     /* line data */
@@ -158,13 +176,9 @@ read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     Handle *h1, *h2;
     
     DiaObject *line_obj;
-    Color line_colour = { 0.0, 0.0, 0.0 };
-    RGB_t color;
+    Color line_colour;
+    RGB_t color = { 0, };
     GPtrArray *props;
-    PointProperty *ptprop;
-    LinestyleProperty *lsprop;
-    ColorProperty *cprop;
-    RealProperty *rprop;
 
     real line_width = DEFAULT_LINE_WIDTH;
     LineStyle style = LINESTYLE_SOLID;
@@ -173,6 +187,8 @@ read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     end.x=0;
     end.y=0;
 
+    props = g_ptr_array_new();
+
     do {
         if(read_dxf_codes(filedxf, data) == FALSE){
             return( NULL );
@@ -180,7 +196,9 @@ read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
         switch(data->code){
         case 6:	 style = get_dia_linestyle_dxf(data->value);
             break;		
-        case  8: layer = layer_find_by_name(data->value, dia);
+        case  8: 
+	    layer = layer_find_by_name(data->value, dia);
+	    color = pal_get_rgb (_dxf_color_get_by_layer (layer));
             break;
         case 10:
             start.x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
@@ -200,34 +218,19 @@ read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
             break;
 	case 62 :
             color = pal_get_rgb (atoi(data->value));
-	    line_colour.red = color.r / 255.0;
-	    line_colour.green = color.g / 255.0;
-	    line_colour.blue = color.b / 255.0;
             break;
         }	
     } while(data->code != 0);
 
+    _color_init_from_rgb (&line_colour, color);
     line_obj = otype->ops->create(&start, otype->default_user_data,
                                   &h1, &h2);
-		
-    props = prop_list_from_descs(dxf_prop_descs,pdtpp_true);
-    g_assert(props->len == 5);
 
-    ptprop = g_ptr_array_index(props,0);
-    ptprop->point_data = start;
-
-    ptprop = g_ptr_array_index(props,1);
-    ptprop->point_data = end;
-
-    cprop = g_ptr_array_index(props,2);
-    cprop->color_data = line_colour;
-
-    rprop = g_ptr_array_index(props,3);
-    rprop->real_data = line_width;
-
-    lsprop = g_ptr_array_index(props,4);
-    lsprop->style = style;
-    lsprop->dash = 1.0;
+    prop_list_add_point (props, "start_point", &start);
+    prop_list_add_point (props, "end_point", &end);
+    prop_list_add_line_colour (props, &line_colour);
+    prop_list_add_line_width (props, line_width);
+    prop_list_add_line_style (props, style, 1.0);
 
     line_obj->ops->set_props(line_obj, props);
 
@@ -241,16 +244,8 @@ read_entity_line_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     return NULL;
 }
 
-static PropDescription dxf_solid_prop_descs[] = {
-     { "line_colour", PROP_TYPE_COLOUR },
-     { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH },
-     { "line_style", PROP_TYPE_LINESTYLE },
-     { "fill_colour", PROP_TYPE_COLOUR },
-     { "show_background", PROP_TYPE_BOOL },
-   PROP_DESC_END};
-
 /* reads a solid entity from the dxf file and creates a polygon object in dia*/
-DiaObject *
+static DiaObject *
 read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
    /* polygon data */
@@ -262,21 +257,19 @@ read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
    DiaObject *polygon_obj;
    MultipointCreateData *pcd;
 
-   Color fill_colour = { 0.5, 0.5, 0.5 };
+   Color fill_colour;
 
    GPtrArray *props;
-   LinestyleProperty *lsprop;
-   ColorProperty *cprop, *fprop;
-   RealProperty *rprop;
-   BoolProperty *bprop;
    
-    real line_width = 0.001;
-    LineStyle style = LINESTYLE_SOLID;
-    Layer *layer = dia->active_layer;
-   RGB_t color;
+   real line_width = 0.001;
+   LineStyle style = LINESTYLE_SOLID;
+   Layer *layer = dia->active_layer;
+   RGB_t color  = { 127, 127, 127 };
    
 /*   printf( "Solid " ); */
 
+   memset(p, 0, sizeof(p));
+
     do {
         if(read_dxf_codes(filedxf, data) == FALSE){
             return( NULL );
@@ -287,6 +280,7 @@ read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 	   break;		
         case  8: 
 	   layer = layer_find_by_name(data->value, dia);
+	   color = pal_get_rgb (_dxf_color_get_by_layer (layer));
 	   /*printf( "layer: %s ", data->value );*/
 	   break;
         case 10:
@@ -297,12 +291,12 @@ read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
             p[1].x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
 	   /*printf( "P1.x: %f ", p[1].x );*/
             break;
-        case 12: 
-            p[2].x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
+        case 12: /* bot only swapped, but special for only 3 points given */
+            p[2].x = p[3].x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
 	   /*printf( "P2.x: %f ", p[2].x );*/
             break;
-        case 13: 
-            p[3].x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
+        case 13: /* SOLID order swapped compared to Dia's */
+            p[2].x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
 	   /*printf( "P3.x: %f ", p[3].x );*/
             break;
         case 20: 
@@ -314,11 +308,11 @@ read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 	   /*printf( "P1.y: %f ", p[1].y );*/
             break;
         case 22: 
-            p[2].y = (-1)*g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
+            p[2].y = p[3].y = (-1)*g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
 	   /*printf( "P2.y: %f ", p[2].y );*/
             break;
         case 23: 
-            p[3].y = (-1)*g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
+            p[2].y = (-1)*g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
 	   /*printf( "P3.y: %f\n", p[3].y );*/
             break;
         case 39: 
@@ -327,9 +321,6 @@ read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
             break;
         case 62: 
             color = pal_get_rgb (atoi(data->value));
-	    fill_colour.red = color.r / 255.0;
-	    fill_colour.green = color.g / 255.0;
-	    fill_colour.blue = color.b / 255.0;
             break;
         }	
     } while(data->code != 0);
@@ -347,24 +338,15 @@ read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 
    polygon_obj = otype->ops->create( NULL, pcd, &h1, &h2 );
 
-   props = prop_list_from_descs( dxf_solid_prop_descs, pdtpp_true );
-   g_assert(props->len == 5);
+    _color_init_from_rgb (&fill_colour, color);
 
-   cprop = g_ptr_array_index( props,0 );
-   cprop->color_data = fill_colour;
+   props = g_ptr_array_new ();
 
-   rprop = g_ptr_array_index( props,1 );
-   rprop->real_data = line_width;
-
-   lsprop = g_ptr_array_index( props,2 );
-   lsprop->style = style;
-   lsprop->dash = 1.0;
-
-   fprop = g_ptr_array_index( props, 3 );
-   fprop->color_data = fill_colour;
-
-   bprop = g_ptr_array_index( props, 4 );
-   bprop->bool_data = TRUE;
+   prop_list_add_line_colour (props, &fill_colour);
+   prop_list_add_line_width (props, line_width);
+   prop_list_add_line_style (props, style, 1.0);
+   prop_list_add_fill_colour (props, &fill_colour);
+   prop_list_add_show_background (props, TRUE);
 
    polygon_obj->ops->set_props( polygon_obj, props );
 
@@ -378,13 +360,8 @@ read_entity_solid_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
    return NULL;
 }
 
-static PropDescription dxf_polyline_prop_descs[] = {
-     { "line_colour", PROP_TYPE_COLOUR },
-     { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH },
-     { "line_style", PROP_TYPE_LINESTYLE },
-   PROP_DESC_END};
-
-static int is_equal( double a, double b )
+static int
+is_equal( double a, double b )
 {
    double epsilon = 0.00001;
    
@@ -398,7 +375,7 @@ static int is_equal( double a, double b )
 }
 
 /* reads a polyline entity from the dxf file and creates a polyline object in dia*/
-DiaObject *
+static DiaObject *
 read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
     int i;
@@ -412,18 +389,15 @@ read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     DiaObject *polyline_obj;
     MultipointCreateData *pcd;
 
-    Color line_colour = { 0.0, 0.0, 0.0 };
+    Color line_colour;
 
     GPtrArray *props;
-    LinestyleProperty *lsprop;
-    ColorProperty *cprop;
-    RealProperty *rprop;
-   
+
     real line_width = DEFAULT_LINE_WIDTH;
     real radius, start_angle = 0;
     LineStyle style = LINESTYLE_SOLID;
     Layer *layer = dia->active_layer;
-    RGB_t color;
+    RGB_t color = { 0, };
     unsigned char closed = 0;
     int points = 0;
     real bulge = 0.0;
@@ -451,6 +425,7 @@ read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
                 break;		
             case  8: 
                 layer = layer_find_by_name(data->value, dia);
+	        color = pal_get_rgb (_dxf_color_get_by_layer (layer));
                     /*printf( "layer: %s ", data->value );*/
                 break;
             case 10:
@@ -486,9 +461,6 @@ read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
                 break;
             case 62: 
                 color = pal_get_rgb (atoi(data->value));
-                line_colour.red = color.r / 255.0;
-                line_colour.green = color.g / 255.0;
-                line_colour.blue = color.b / 255.0;
                 break;
             case 70:
                 closed = 1 & atoi( data->value );
@@ -589,18 +561,13 @@ read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 
     polyline_obj = otype->ops->create( NULL, pcd, &h1, &h2 );
 
-    props = prop_list_from_descs( dxf_polyline_prop_descs, pdtpp_true );
-    g_assert( props->len == 3 );
+    _color_init_from_rgb (&line_colour, color);
 
-    cprop = g_ptr_array_index( props,0 );
-    cprop->color_data = line_colour;
+    props = g_ptr_array_new ();
 
-    rprop = g_ptr_array_index( props,1 );
-    rprop->real_data = line_width;
-
-    lsprop = g_ptr_array_index( props,2 );
-    lsprop->style = style;
-    lsprop->dash = 1.0;
+    prop_list_add_line_colour (props, &line_colour);
+    prop_list_add_line_width (props, line_width);
+    prop_list_add_line_style (props, style, 1.0);
 
     polyline_obj->ops->set_props( polyline_obj, props );
 
@@ -614,32 +581,21 @@ read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     return NULL; /* don't add it twice */
 }
 
-static PropDescription dxf_ellipse_prop_descs[] = {
-    { "elem_corner", PROP_TYPE_POINT },
-    { "elem_width", PROP_TYPE_REAL },
-    { "elem_height", PROP_TYPE_REAL },
-    { "line_colour", PROP_TYPE_COLOUR },
-    { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH },
-    { "show_background", PROP_TYPE_BOOL},
-    PROP_DESC_END};
-
 /* reads a circle entity from the dxf file and creates a circle object in dia*/
-DiaObject *read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
+static DiaObject *
+read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
     /* circle data */
-    Point center;
+    Point center = {0, 0};
     real radius = 1.0;
     
     DiaObjectType *otype = object_get_type("Standard - Ellipse");  
     Handle *h1, *h2;
     
     DiaObject *ellipse_obj;
-    Color line_colour = { 0.0, 0.0, 0.0 };
+    RGB_t color  = { 0, };
+    Color line_colour;
 
-    PointProperty *ptprop;
-    RealProperty *rprop;
-    BoolProperty *bprop;
-    ColorProperty *cprop;
     GPtrArray *props;
 
     real line_width = DEFAULT_LINE_WIDTH;
@@ -652,6 +608,7 @@ DiaObject *read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia
         switch(data->code){
         case  8: 
             layer = layer_find_by_name(data->value, dia);
+	    color = pal_get_rgb (_dxf_color_get_by_layer (layer));
             break;
         case 10: 
             center.x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
@@ -665,6 +622,9 @@ DiaObject *read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia
         case 40: 
             radius = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
             break;
+	case 62 :
+            color = pal_get_rgb (atoi(data->value));
+            break;
         }
         
     } while(data->code != 0);
@@ -674,21 +634,16 @@ DiaObject *read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia
     ellipse_obj = otype->ops->create(&center, otype->default_user_data,
                                      &h1, &h2);
 	
-    props = prop_list_from_descs(dxf_ellipse_prop_descs,pdtpp_true);
-    g_assert(props->len == 6);
+    _color_init_from_rgb (&line_colour, color);
 
-    ptprop = g_ptr_array_index(props,0);
-    ptprop->point_data = center;
-    rprop = g_ptr_array_index(props,1);
-    rprop->real_data = radius * 2.0;
-    rprop = g_ptr_array_index(props,2);
-    rprop->real_data = radius * 2.0;
-    cprop = g_ptr_array_index(props,3);
-    cprop->color_data = line_colour;
-    rprop = g_ptr_array_index(props,4);
-    rprop->real_data = line_width;
-    bprop = g_ptr_array_index(props,5);
-    bprop->bool_data = FALSE;
+    props = g_ptr_array_new ();
+
+    prop_list_add_point (props, "elem_corner", &center);
+    prop_list_add_real (props, "elem_width", radius * 2.0);
+    prop_list_add_real (props, "elem_height", radius * 2.0);
+    prop_list_add_line_colour (props, &line_colour);
+    prop_list_add_line_width (props, line_width);
+    prop_list_add_show_background (props, FALSE);
     
     ellipse_obj->ops->set_props(ellipse_obj, props);
     prop_list_free(props);
@@ -701,19 +656,13 @@ DiaObject *read_entity_circle_dxf(FILE *filedxf, DxfData *data, DiagramData *dia
     return NULL; /* don't add it twice */
 }
 
-static PropDescription dxf_arc_prop_descs[] = {
-    { "start_point", PROP_TYPE_POINT },
-    { "end_point", PROP_TYPE_POINT },
-    { "curve_distance", PROP_TYPE_REAL },
-    { "line_colour", PROP_TYPE_COLOUR },
-    { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH },
-    PROP_DESC_END};
-
 /* reads a circle entity from the dxf file and creates a circle object in dia*/
-DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
+static DiaObject *
+read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
     /* arc data */
-    Point start,center,end;
+    Point start, end;
+    Point center = {0, 0};
     real radius = 1.0, start_angle = 0.0, end_angle=360.0;
     real curve_distance;
 
@@ -721,11 +670,8 @@ DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     Handle *h1, *h2;
   
     DiaObject *arc_obj;
-    Color line_colour = { 0.0, 0.0, 0.0 };
-
-    ColorProperty *cprop;
-    PointProperty *ptprop;
-    RealProperty *rprop;
+    RGB_t color  = { 0, };
+    Color line_colour;
     GPtrArray *props;
 
     real line_width = DEFAULT_LINE_WIDTH;
@@ -738,6 +684,7 @@ DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
         switch(data->code){
         case  8: 
             layer = layer_find_by_name(data->value, dia);
+	    color = pal_get_rgb (_dxf_color_get_by_layer (layer));
             break;
         case 10: 
             center.x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
@@ -757,6 +704,9 @@ DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
         case 51:
             end_angle = g_ascii_strtod(data->value, NULL)*M_PI/180.0;
             break;
+	case 62 :
+            color = pal_get_rgb (atoi(data->value));
+            break;
         }
     } while(data->code != 0);
 
@@ -777,19 +727,14 @@ DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     arc_obj = otype->ops->create(&center, otype->default_user_data,
                                      &h1, &h2);
     
-    props = prop_list_from_descs(dxf_arc_prop_descs,pdtpp_true);
-    g_assert(props->len == 5);
+    _color_init_from_rgb (&line_colour, color);
 
-    ptprop = g_ptr_array_index(props,0);
-    ptprop->point_data = start;
-    ptprop = g_ptr_array_index(props,1);
-    ptprop->point_data = end;
-    rprop = g_ptr_array_index(props,2);
-    rprop->real_data = curve_distance;
-    cprop = g_ptr_array_index(props,3);
-    cprop->color_data = line_colour;
-    rprop = g_ptr_array_index(props,4);
-    rprop->real_data = line_width;
+    props = g_ptr_array_new ();
+    prop_list_add_point (props, "start_point", &start);
+    prop_list_add_point (props, "end_point", &end);
+    prop_list_add_real (props, "curve_distance", curve_distance);
+    prop_list_add_line_colour (props, &line_colour);
+    prop_list_add_line_width (props, line_width);
     
     arc_obj->ops->set_props(arc_obj, props);
     prop_list_free(props);
@@ -803,11 +748,11 @@ DiaObject *read_entity_arc_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* reads an ellipse entity from the dxf file and creates an ellipse object in dia*/
-DiaObject *
+static DiaObject *
 read_entity_ellipse_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
     /* ellipse data */
-    Point center;
+    Point center = {0, 0};
     real width = 1.0;
     real ratio_width_height = 1.0;
     
@@ -815,11 +760,8 @@ read_entity_ellipse_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     Handle *h1, *h2;
     
     DiaObject *ellipse_obj; 
-    Color line_colour = { 0.0, 0.0, 0.0 };
-    PointProperty *ptprop;
-    RealProperty *rprop;
-    BoolProperty *bprop;
-    ColorProperty *cprop;
+    RGB_t color = { 0, };
+    Color line_colour;
     GPtrArray *props;
 
     real line_width = DEFAULT_LINE_WIDTH;
@@ -832,6 +774,7 @@ read_entity_ellipse_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
         switch(data->code){
         case  8: 
             layer = layer_find_by_name(data->value, dia);
+	    color = pal_get_rgb (_dxf_color_get_by_layer (layer));
             break;
         case 10: 
             center.x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
@@ -848,6 +791,9 @@ read_entity_ellipse_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
         case 40: 
             width = g_ascii_strtod(data->value, NULL) * WIDTH_SCALE; /* XXX what scale */
             break;
+	case 62 :
+            color = pal_get_rgb (atoi(data->value));
+            break;
         }
     } while(data->code != 0);
  
@@ -855,22 +801,17 @@ read_entity_ellipse_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     center.y -= (width*ratio_width_height);
     ellipse_obj = otype->ops->create(&center, otype->default_user_data,
                                      &h1, &h2);
-        
-    props = prop_list_from_descs(dxf_ellipse_prop_descs,pdtpp_true);
-    g_assert(props->len == 6);
 
-    ptprop = g_ptr_array_index(props,0);
-    ptprop->point_data = center;
-    rprop = g_ptr_array_index(props,1);
-    rprop->real_data = width;
-    rprop = g_ptr_array_index(props,2);
-    rprop->real_data = width * ratio_width_height;
-    cprop = g_ptr_array_index(props,3);
-    cprop->color_data = line_colour;
-    rprop = g_ptr_array_index(props,4);
-    rprop->real_data = line_width;
-    bprop = g_ptr_array_index(props,5);
-    bprop->bool_data = FALSE;
+    _color_init_from_rgb (&line_colour, color);
+
+    props = g_ptr_array_new ();
+
+    prop_list_add_point (props, "elem_corner", &center);
+    prop_list_add_real (props, "elem_width", width);
+    prop_list_add_real (props, "elem_height", width * ratio_width_height);
+    prop_list_add_line_colour (props, &line_colour);
+    prop_list_add_line_width (props, line_width);
+    prop_list_add_show_background (props, FALSE);
     
     ellipse_obj->ops->set_props(ellipse_obj, props);
     prop_list_free(props);
@@ -887,13 +828,13 @@ static PropDescription dxf_text_prop_descs[] = {
     { "text", PROP_TYPE_TEXT },
     PROP_DESC_END};
 
-DiaObject *
+static DiaObject *
 read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
-    RGB_t color;
+    RGB_t color = { 0, };
 
     /* text data */
-    Point location;
+    Point location = {0, 0};
     real height = text_scale * coord_scale * measure_scale;
     real y_offset = 0;
     Alignment textalignment = ALIGN_LEFT;
@@ -903,7 +844,7 @@ read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     Handle *h1, *h2;
     
     DiaObject *text_obj;
-    Color text_colour = { 0.0, 0.0, 0.0 };
+    Color text_colour;
 
     TextProperty *tprop;
     GPtrArray *props;
@@ -933,7 +874,9 @@ read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 		
 	   /*printf( "Found text: %s\n", textvalue );*/
             break;
-        case  8: layer = layer_find_by_name(data->value, dia);
+        case  8: 
+	    layer = layer_find_by_name(data->value, dia);
+	    color = pal_get_rgb (_dxf_color_get_by_layer (layer));
             break;
         case 10: 
             location.x = g_ascii_strtod(data->value, NULL) * coord_scale * measure_scale;
@@ -957,10 +900,7 @@ read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 	   /*printf( "text height %f\n", height );*/
             break;
 	 case 62: 
-	   color = pal_get_rgb (atoi(data->value));
-	   text_colour.red = color.r / 255.0;
-	   text_colour.green = color.g / 255.0;
-	   text_colour.blue = color.b / 255.0;
+	    color = pal_get_rgb (atoi(data->value));
             break;
         case 72: 
 	   switch(atoi(data->value))
@@ -1009,7 +949,7 @@ read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     } while(data->code != 0);
   
     location.y += y_offset * height;
-   
+    _color_init_from_rgb (&text_colour, color);
     text_obj = otype->ops->create(&location, otype->default_user_data,
                                   &h1, &h2);
     props = prop_list_from_descs(dxf_text_prop_descs,pdtpp_true);
@@ -1038,23 +978,32 @@ read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* reads the layer table from the dxf file and creates the layers */
-void 
+static void
 read_table_layer_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
-    do {
-        if(read_dxf_codes(filedxf, data) == FALSE){
-            return;
-        }
-        else {
-            if(data->code == 2){
-	       layer_find_by_name( data->value, dia );
-            }
-        }
-    } while ((data->code != 0) || (strcmp(data->value, "ENDTAB") != 0));
+     Layer *layer = NULL;
+     int color_index;
+
+     do {
+	if (read_dxf_codes(filedxf, data) == FALSE)
+	    return;
+	switch (data->code) {
+	case 2 : /* layer name */
+	    layer = layer_find_by_name( data->value, dia );
+	    break;
+	case 62 : /* Color number, if negative layer is off */
+	    color_index = atoi(data->value);
+	    if (layer && color_index < 0)
+		layer->visible = FALSE;
+	    else
+	        _dxf_color_set_by_layer (layer, color_index);
+	    break;
+	} 
+     } while ((data->code != 0) || (strcmp(data->value, "ENDTAB") != 0));
 }
 
 /* reads a scale entity from the dxf file */
-void 
+static void
 read_entity_scale_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
    if(read_dxf_codes(filedxf, data) == FALSE)
@@ -1064,6 +1013,8 @@ read_entity_scale_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
      {
       case 40: 
 	coord_scale = g_ascii_strtod(data->value, NULL);
+	if (coord_scale > 0.0)
+	  coord_scale = 1.0 / coord_scale;
 	g_message("Scale: %f", coord_scale );
 	break;
       
@@ -1073,9 +1024,33 @@ read_entity_scale_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
    
 }
 
+static void
+read_entitiy_lengthunit_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
+{
+   real e; /* undocumented ... */
+
+   if(read_dxf_codes(filedxf, data) == FALSE)
+      return;
+
+   switch(data->code)
+     {
+      case 70: 
+	/* 1 = Scientific; 2 = Decimal; 3 = Engineering;
+         * 4 = Architectural; 5 = Fractional; 6 = Windows desktop 
+	 */
+	e = atoi(data->value);
+	g_message("LengthUnit: %f:%f", e, coord_scale );
+	break;
+      
+      default:
+	break;
+     }	
+   
+}
+
 /* reads a scale entity from the dxf file */
-void read_entity_measurement_dxf(FILE *filedxf, DxfData *data,
-                                 DiagramData *dia)
+static void 
+read_entity_measurement_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
    if(read_dxf_codes(filedxf, data) == FALSE)
       return;
@@ -1098,7 +1073,7 @@ void read_entity_measurement_dxf(FILE *filedxf, DxfData *data,
 }
 
 /* reads a textsize entity from the dxf file */
-void 
+static void
 read_entity_textsize_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 {
    if(read_dxf_codes(filedxf, data) == FALSE)
@@ -1108,6 +1083,8 @@ read_entity_textsize_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
      {
       case 40:
 	text_scale = g_ascii_strtod(data->value, NULL);
+	if (text_scale > 0.0)
+	  text_scale = 1.0 / text_scale;
 	/*printf( "Text Size: %f\n", text_scale );*/
 	break;		
       default:
@@ -1117,7 +1094,7 @@ read_entity_textsize_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* reads the headers section of the dxf file */
-void 
+static void
 read_section_header_dxf(FILE *filedxf, DxfData *data, DiagramData *dia) 
 {
     if(read_dxf_codes(filedxf, data) == FALSE){
@@ -1126,6 +1103,9 @@ read_section_header_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     do {
        if((data->code == 9) && (strcmp(data->value, "$DIMSCALE") == 0)) {
 	  read_entity_scale_dxf(filedxf, data, dia);
+	} else if((data->code == 9) && (strcmp(data->value, "DIMLUNIT") == 0)) {
+	  /* nothing documented */
+	  read_entitiy_lengthunit_dxf(filedxf, data, dia);
         } else if((data->code == 9) && (strcmp(data->value, "$TEXTSIZE") == 0)) {
 	  read_entity_textsize_dxf(filedxf, data, dia);
         } else if((data->code == 9) && (strcmp(data->value, "$MEASUREMENT") == 0)) {
@@ -1140,7 +1120,7 @@ read_section_header_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* reads the classes section of the dxf file */
-void 
+static void
 read_section_classes_dxf(FILE *filedxf, DxfData *data, DiagramData *dia) 
 {
     if(read_dxf_codes(filedxf, data) == FALSE){
@@ -1161,7 +1141,7 @@ read_section_classes_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* reads the tables section of the dxf file */
-void 
+static void
 read_section_tables_dxf(FILE *filedxf, DxfData *data, DiagramData *dia) 
 {
     if(read_dxf_codes(filedxf, data) == FALSE){
@@ -1180,7 +1160,7 @@ read_section_tables_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* reads the entities section of the dxf file */
-void 
+static void
 read_section_entities_dxf(FILE *filedxf, DxfData *data, DiagramData *dia) 
 {
     if (read_dxf_codes(filedxf, data) == FALSE){
@@ -1212,7 +1192,7 @@ read_section_entities_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* reads the blocks section of the dxf file */
-void 
+static void
 read_section_blocks_dxf(FILE *filedxf, DxfData *data, DiagramData *dia) 
 {
     int group_items = 0, group = 0;
@@ -1298,16 +1278,16 @@ read_section_blocks_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* imports the given dxf-file, returns TRUE if successful */
-gboolean
-import_dxf(const gchar *filename, DiagramData *dia, void* user_data)
+static gboolean
+import_dxf(const gchar *filename, DiagramData *dia, DiaContext *ctx, void* user_data)
 {
     FILE *filedxf;
     DxfData *data;
     
     filedxf = g_fopen(filename,"r");
     if(filedxf == NULL){
-        message_error(_("Couldn't open: '%s' for reading.\n"), 
-		      dia_message_filename(filename));
+        dia_context_add_message(ctx, _("Couldn't open: '%s' for reading.\n"),
+				dia_context_get_filename (ctx));
         return FALSE;
     }
     
@@ -1316,15 +1296,15 @@ import_dxf(const gchar *filename, DiagramData *dia, void* user_data)
     do {
         if(read_dxf_codes(filedxf, data) == FALSE) {
             g_free(data);
-	    message_error(_("read_dxf_codes failed on '%s'\n"),
-			  dia_message_filename(filename) );
+	    dia_context_add_message(ctx, _("read_dxf_codes failed on '%s'"),
+			            dia_context_get_filename(ctx) );
             return FALSE;
         }
         else {
             if (0 == data->code && strstr(data->codeline, "AutoCAD Binary DXF")) {
                 g_free(data);
-	        message_error(_("Binary DXF from '%s' not supported\n"),
-			      dia_message_filename(filename) );
+	        dia_context_add_message(ctx, _("Binary DXF from '%s' not supported"),
+			                dia_context_get_filename(ctx));
                 return FALSE;
             }
 	    if (0 == data->code) {
@@ -1362,6 +1342,8 @@ import_dxf(const gchar *filename, DiagramData *dia, void* user_data)
 		  /*printf( "reading section objects\n" );*/
                     read_section_entities_dxf(filedxf, data, dia);
 		}
+            } else if(data->code == 999) {
+	      /* Don't complain on comments, but silently ignore */
 	    }
 	   else
 	     g_warning("Unknown dxf code %d", data->code);
@@ -1369,12 +1351,17 @@ import_dxf(const gchar *filename, DiagramData *dia, void* user_data)
     }while((data->code != 0) || (strcmp(data->value, "EOF") != 0));
     
     g_free(data);
-    
+    if (_color_by_layer_ht) {
+        g_hash_table_destroy (_color_by_layer_ht);
+        _color_by_layer_ht = NULL;
+    }
     return TRUE;
 }
 
 /* reads a code/value pair from the DXF file */
-gboolean read_dxf_codes(FILE *filedxf, DxfData *data){
+static gboolean 
+read_dxf_codes(FILE *filedxf, DxfData *data)
+{
     int i;
     char *c;
     
